/*****************************************************************
/
/ File   :   T0Hndlr.c
/ Author :   David Corcoran
/ Date   :   October 15, 1999
/ Purpose:   This provides a T=0 handler.
/            See http://www.linuxnet.com for more information.
/ License:   See file LICENSE
/
******************************************************************/

#include "T0Hndlr.h"
#include "AdmHndlr.h"
#include "serial.h"

ULONG T0_ExchangeData( ULONG ulSlot, PUCHAR pRequest, ULONG RequestLen, 
                       PUCHAR pReply, PULONG pReplyLen ) {

  ULONG rv;
  UCHAR ucTxBuffer[MAX_BUFFER_SIZE]; /* For byte splitting */
  UCHAR ucResponse[MAX_BUFFER_SIZE];
  ULONG ulRxLength;

  switch( ulSlot ) {
  case 0x01:
    ucTxBuffer[1] = 0x03;
    break;
  case 0x02:
    ucTxBuffer[1] = 0x02;
    break;
  default:
    return STATUS_UNSUCCESSFUL;
  }

  if ( RequestLen > 5 ) { /* Must be a write function */
    ucTxBuffer[0] = 0x23;
  } else {                /* Must be a read function  */
    ucTxBuffer[0] = 0x24;
  }
  
  memcpy( &ucTxBuffer[2], pRequest, RequestLen );
  
  rv = Adm_Transmit( ucTxBuffer, RequestLen + 2, ucResponse, &ulRxLength );

  if ( rv == STATUS_SUCCESS ) {
    memcpy( pReply, &ucResponse[2], ulRxLength - 3 );
    memcpy( &pReply[ulRxLength - 3], ucResponse, 2 );
    *pReplyLen = ulRxLength - 1;
  } else {
    *pReplyLen = 0;
  }


  return rv;

}


