/*****************************************************************
/
/ File     :   ifdhandler.c
/ Author   :   David Corcoran
/ Date     :   September 15, 1999
/ Modified :
/ Purpose  :   This provides reader specific low-level calls.
/              See http://www.linuxnet.com for more information.
/ License  :   See file LICENSE
/
******************************************************************/

#include "ifdhandler.h"
#include "T0Hndlr.h"
#include "T1Hndlr.h"
#include <string.h>

/* This is a quick fix to support multiple
   slots on the same terminal                */

BYTE ICCATRa[MAX_ATR_SIZE];
BYTE ICCATRb[MAX_ATR_SIZE];

static unsigned char isInitialized = 0;

RESPONSECODE IFDHCreateChannel ( DWORD Lun, DWORD Channel ) {

  RESPONSECODE rv = IFD_SUCCESS;

  char *pcPortId;

  unsigned short Unibble;         /* Upper 16bits of DWORD. (DDDD) */
  unsigned short Lnibble;         /* Lower 16bits of DWORD. (CCCC) */

  Unibble = Channel >> 16;                 /* Shift right 16 bits.     */
  Lnibble = Channel - ( Unibble << 16 );   /* Shift left and subtract. */

  switch ( Unibble ) {

    case 0x01:                               /* Serial Port.             */

      switch ( Lnibble ) {

        case 0x3F8:
          pcPortId = strdup("/dev/ttyS0");
          break;

        case 0x2F8:
          pcPortId = strdup("/dev/ttyS1");
          break;

        case 0x3E8:
           pcPortId = strdup("/dev/ttyS2");
          break;

        case 0x2E8:
           pcPortId = strdup("/dev/ttyS3");
          break;
    
        default:
          rv = IFD_NOT_SUPPORTED;
          break;
      }


      break;

    case 0x02:                               /* Parallel Port.     */
      rv = IFD_NOT_SUPPORTED;
      break;

    case 0x04:                               /* PS 2 Port.         */
      rv = IFD_NOT_SUPPORTED;
      break;

    case 0x08:                               /* SCSI Port.         */
      rv = IFD_NOT_SUPPORTED;
      break;

    case 0x10:                               /* IDE Port.          */
      rv = IFD_NOT_SUPPORTED;
      break;

    case 0x20:                               /* USB Port.          */
      rv = IFD_NOT_SUPPORTED;
      break;

    /* case 0xFy where y is vendor defined 0 - F is NOT implemented */

    default:                                 /* Port Not Found.    */
      rv = IFD_NOT_SUPPORTED;
      break;

  }
  
  if ( rv == IFD_SUCCESS && isInitialized == 0 ) {
      if ( Adm_Initialize(pcPortId) == STATUS_SUCCESS ) {
	  
	  if ( Adm_PowerIFD() == STATUS_SUCCESS ) {
	    isInitialized = 1;
	    rv = IFD_SUCCESS;
	  } else {
	    rv = IFD_COMMUNICATION_ERROR;
	  }
      } else {
	rv = IFD_COMMUNICATION_ERROR;
      }
  }
  
  free( pcPortId );
  return rv;
}

RESPONSECODE IFDHCloseChannel ( DWORD Lun ) {

  RESPONSECODE rv    = IFD_SUCCESS;
  unsigned int iTerm = 0;


  if ( isInitialized == 1 ) {
    Adm_UnPowerIFD();
    if ( Adm_UnInitialize() == STATUS_SUCCESS ) {
      isInitialized = 0;
      rv = IFD_SUCCESS;
    } else {
      rv = IFD_COMMUNICATION_ERROR;
    }
  }

  return rv;
}


RESPONSECODE IFDHGetCapabilities ( DWORD Lun, DWORD Tag, 
				   PDWORD Length, PUCHAR Value ) {

  switch(Tag) {
    /* Number of slots the reader has */
  case TAG_IFD_SLOTS_NUMBER:
    Value[0] = 1;
    *Length  = 1;
    break;
    /* Number of simultaneous readers supported */
  case TAG_IFD_SIMULTANEOUS_ACCESS:
    Value[0] = 1;
    *Length  = 1;
    break;
  default:
    return IFD_ERROR_TAG;
  }
  
  return IFD_SUCCESS;
}

RESPONSECODE IFDHSetCapabilities ( DWORD Lun, DWORD Tag, 
				   DWORD Length, PUCHAR Value ) {

  /* No parameters to set */
  return IFD_ERROR_TAG;
}

RESPONSECODE IFDHSetProtocolParameters ( DWORD Lun, DWORD Protocol, 
					 UCHAR Flags, UCHAR PTS1,
					 UCHAR PTS2, UCHAR PTS3) {
  RESPONSECODE rv;
  DWORD        PTSCount;
  UCHAR	       pucPTSRequest[6], pucPTSReply[6];
  ULONG	       NewProtocol, BufferLength;
  int i;

#ifdef PCSC_DEBUG
  printf("IFD Sublayer reader to set protocol %d\n", Protocol);
#endif

  PTSCount        = 0;

  /* Default PTSS             */
  pucPTSRequest[0] = 0xFF;
  
  /* set the format character */
  if ( Protocol == 1 ) {
    pucPTSRequest[1] = (Flags*0x10) + 0x01;
  } else {
    pucPTSRequest[1] = (Flags*0x10) + 0x00;
  }
  
  PTSCount = 2;

  if ( Flags & 0x01 ) {
    pucPTSRequest[2] = PTS1;
    PTSCount += 1;
  }
  if ( Flags & 0x02 ) {
    pucPTSRequest[3] = PTS2;
    PTSCount += 1;
  }
  if ( Flags & 0x04 ) {
    pucPTSRequest[4] = PTS3;
    PTSCount += 1;
  }

  /*	check character  */

  pucPTSRequest[PTSCount] = 0x00;

  for ( i=0; i < PTSCount; i++ ) {
    pucPTSRequest[PTSCount] ^= pucPTSRequest[i];
  }  

  /*	write PTSRequest */

  /* FILL IN CODE TO WRITE OUT PTS REQUEST */
  
  /*	get response     */

  /* FILL IN CODE TO READ IN PTS RESPONSE */

  return IFD_ERROR_PTS_FAILURE;

#ifdef PCSC_DEBUG
      printf("PTS Response: ");
      for ( i=0; i < PTSCount; i++ ) {
	printf("%x ", pucPTSReply[i]);
      } printf("\n");
#endif

  
  if ( memcmp(pucPTSReply, pucPTSRequest, PTSCount) != 0 ) {
    rv = IFD_ERROR_PTS_FAILURE;
  } else {
    rv = IFD_SUCCESS;
  }

  return rv;
}

RESPONSECODE IFDHPowerICC ( DWORD Lun, DWORD Action, 
			    PUCHAR Atr, PDWORD AtrLength ) {
  
  RESPONSECODE rv;
  UCHAR ucAtr[MAX_ATR_SIZE];
  DWORD dwAtrLen;
  DWORD dwSlotID;
  int i;

  dwAtrLen = MAX_ATR_SIZE;
  dwSlotID = Lun & 0x00FF;
  dwSlotID += 1;

  if ( Action == IFD_POWER_UP ) {
    rv = Adm_PowerICC( dwSlotID, ucAtr, &dwAtrLen );
      if ( rv == STATUS_SUCCESS ) {
	if ( dwSlotID == 1 ) {
	  memcpy( ICCATRa, ucAtr, dwAtrLen );         /* Copies the ATR. */
	  memcpy(Atr, ucAtr, dwAtrLen);
	  *AtrLength = dwAtrLen;
	} else {
	  memcpy( ICCATRb, ucAtr, dwAtrLen );
	  memcpy(Atr, ucAtr, dwAtrLen);
	  *AtrLength = dwAtrLen;
	}

        rv = IFD_SUCCESS;
      } else {
        rv = IFD_ERROR_POWER_ACTION;
      }
    
  } else if ( Action == IFD_POWER_DOWN ) {
    rv = Adm_UnPowerICC( dwSlotID );
    if ( rv == STATUS_SUCCESS ) {
      if ( dwSlotID == 1 ) {
	memset( ICCATRa, 0x00, MAX_ATR_SIZE );
	*AtrLength = 0;
      } else {
	memset( ICCATRb, 0x00, MAX_ATR_SIZE );
	*AtrLength = 0;
      }
      rv = IFD_SUCCESS;
    } else {
      rv = IFD_ERROR_POWER_ACTION;
    }
    
  } else if ( Action == IFD_RESET ) {
    rv = Adm_ResetICC( dwSlotID, ucAtr, &dwAtrLen );
    if ( rv == STATUS_SUCCESS ) {
      if ( dwSlotID == 1 ) {
	memcpy( ICCATRa, ucAtr, dwAtrLen );         /* Copies the ATR. */
	memcpy(Atr, ucAtr, dwAtrLen);
	*AtrLength = dwAtrLen;
      } else {
	memcpy( ICCATRb, ucAtr, dwAtrLen );
	memcpy(Atr, ucAtr, dwAtrLen);
	*AtrLength = dwAtrLen;
      }
      rv = IFD_SUCCESS;
    } else {
      rv = IFD_ERROR_POWER_ACTION;
    }  
    
  } else {
    rv = IFD_NOT_SUPPORTED;
  }
  
  return rv; 
}


RESPONSECODE IFDHTransmitToICC ( DWORD Lun, SCARD_IO_HEADER SendPci, 
				 PUCHAR TxBuffer, DWORD TxLength, 
				 PUCHAR RxBuffer, PDWORD RxLength, 
				 PSCARD_IO_HEADER RecvPci ) {
  
  RESPONSECODE rv;
  DWORD dwSlotID;
  int i;

  dwSlotID = Lun & 0x00FF;
  dwSlotID += 1;

  /* Cannot send/receive commands larger than 255 bytes 
     these should be split up into 255 byte blocks      */

  if ( TxLength > 260 || *RxLength > 257 ) {
      *RxLength = 0;
      return IFD_ERROR_NOT_SUPPORTED;
  }

  if ( TxLength == 4 ) {  /* Case 1 command caught */
    TxBuffer[4] = 0x00;   /* Add trailing zero     */
    TxLength    = 5; 
  } else if ( TxLength < 4 ) {
    *RxLength = 0;
    return IFD_ERROR_NOT_SUPPORTED;
  }

#ifdef PCSC_DEBUG
  printf("[%04x] -> ", TxLength);
  for ( i=0; i < TxLength; i++ ) {
    printf("%02x ", TxBuffer[i]);
  } printf("\n");
#endif  


    if ( SendPci.Protocol        == 0 ) {
      rv = T0_ExchangeData( dwSlotID, TxBuffer, TxLength, 
                            RxBuffer, RxLength );   

    } else if ( SendPci.Protocol == 1 ) {

      rv = T1_ExchangeData( dwSlotID, TxBuffer, TxLength,
                            RxBuffer, RxLength );
    }

    if ( rv == STATUS_SUCCESS ) {
      rv = IFD_SUCCESS;
      
      /* Print Receive Error Messaging */
#ifdef PCSC_DEBUG
      printf("[%04x] <- ", *RxLength );
      for ( i=0; i < *RxLength; i++ ) {
	printf("%02x ", RxBuffer[i]);
      } printf("\n");
#endif  
      
    } else {
      *RxLength = 0;
      rv = IFD_COMMUNICATION_ERROR;
    }
    
    return rv;    
}
    
RESPONSECODE IFDHControl ( DWORD Lun, PUCHAR TxBuffer, 
			 DWORD TxLength, PUCHAR RxBuffer, 
			 PDWORD RxLength ) {

    return IFD_SUCCESS;
}


RESPONSECODE IFDHICCPresence( DWORD Lun ) {

  RESPONSECODE rv;
  BYTE bStatus;
  DWORD dwSlotID;


  dwSlotID = Lun & 0x00FF;
  dwSlotID += 1;

  if ( Adm_IsICCPresent( dwSlotID ) == STATUS_SUCCESS ) { 
    /* Smartcard detected */
    return IFD_ICC_PRESENT; 
  } else {
    return IFD_ICC_NOT_PRESENT;
  }

  return IFD_COMMUNICATION_ERROR;
}

